// This file was originally part of the project "LURE - Linux User REpository", created by Elara Musayelyan.
// It has been modified as part of "ALR - Any Linux Repository" by Евгений Храмов.
//
// ALR - Any Linux Repository
// Copyright (C) 2025 Евгений Храмов
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

package main

import (
	"log/slog"
	"os"
	"path/filepath"

	"github.com/leonelquinteros/gotext"
	"github.com/urfave/cli/v2"

	"gitea.plemya-x.ru/Plemya-x/ALR/internal/config"
	database "gitea.plemya-x.ru/Plemya-x/ALR/internal/db"
	"gitea.plemya-x.ru/Plemya-x/ALR/internal/utils"
	"gitea.plemya-x.ru/Plemya-x/ALR/pkg/repos"
)

func FixCmd() *cli.Command {
	return &cli.Command{
		Name:  "fix",
		Usage: gotext.Get("Attempt to fix problems with ALR"),
		Action: func(c *cli.Context) error {
			err := utils.DropCapsToAlrUser()
			if err != nil {
				slog.Error(gotext.Get("Can't drop privileges"))
				return cli.Exit(err, 1)
			}

			ctx := c.Context
			cfg := config.New()
			err = cfg.Load()
			if err != nil {
				slog.Error(gotext.Get("Error loading config"))
				return cli.Exit(err, 1)
			}

			paths := cfg.GetPaths()

			slog.Info(gotext.Get("Clearing cache directory"))
			// Remove all nested directories of paths.CacheDir

			dir, err := os.Open(paths.CacheDir)
			if err != nil {
				slog.Error(gotext.Get("Unable to open cache directory"))
				return cli.Exit(err, 1)
			}
			defer dir.Close()

			entries, err := dir.Readdirnames(-1)
			if err != nil {
				slog.Error(gotext.Get("Unable to read cache directory contents"))
				return cli.Exit(err, 1)
			}

			for _, entry := range entries {
				err = os.RemoveAll(filepath.Join(paths.CacheDir, entry))
				if err != nil {
					slog.Error(gotext.Get("Unable to remove cache item"), "item", entry)
					return cli.Exit(err, 1)
				}
			}

			slog.Info(gotext.Get("Rebuilding cache"))

			err = os.MkdirAll(paths.CacheDir, 0o755)
			if err != nil {
				slog.Error(gotext.Get("Unable to create new cache directory"))
				return cli.Exit(err, 1)
			}

			cfg = config.New()
			err = cfg.Load()
			if err != nil {
				slog.Error(gotext.Get("Error loading config"))
				return cli.Exit(err, 1)
			}

			db := database.New(cfg)
			err = db.Init(ctx)
			if err != nil {
				slog.Error(gotext.Get("Error initialization database"))
				return cli.Exit(err, 1)
			}
			rs := repos.New(cfg, db)
			err = rs.Pull(ctx, cfg.Repos())
			if err != nil {
				slog.Error(gotext.Get("Error pulling repos"))
				return cli.Exit(err, 1)
			}

			slog.Info(gotext.Get("Done"))

			return nil
		},
	}
}
